// Fragment Shader of uberlight-like model

// Super ellipse shaping parameters
uniform float a;             //inner superellipse width
uniform float b;             //inner superellipse height
uniform float A;             //outer superellipse width 
uniform float B;             //outer superellipse hight
uniform float d;             //roundness of shape

// Distance attenuation parameters
uniform float zNear;
uniform float zFar;
uniform float nearEdge;
uniform float farEdge;

varying vec3 vPosLC;         // vertex position in light coordinates
varying vec3 normLC;         // normal in light coordinates
varying vec3 eyeLC;          // view position in light coordinates

float superEllipseShape(vec3 pos)
{
   // Project point onto z = 1.0 plane, and consider absolute values
   float x = abs(pos.x/pos.z);
   float y = abs(pos.y/pos.z);

   float e1 = 2.0 / d, e2 = -d / 2.0;

   float q = a * b * pow(pow(b * x, e1) + pow(a * y, e1), e2);
   float r = A * B * pow(pow(B * x, e1) + pow(A * y, e1), e2);

   return 1.0 - smoothstep(q, r, 1.0);
}

float distanceAttenuation(vec3 pos, float maxIntensity, float d0, float alpha )
{
   float z = abs(pos.z);

   // attenuation in various light zones (Figure 20-16)
   float a =  smoothstep(zNear - nearEdge, zNear, z) *
                (1.0 - smoothstep(zFar, zFar + farEdge, z));

   // attenuation due to distance 
   if ( z > d0 )
     a *= pow ( d0/z, alpha );
   else { 
     float s = log ( 1.0 / maxIntensity );
     float beta = -alpha / s;
     a *=  maxIntensity * exp ( s * pow(z/d0, beta) );
   }  

   return a;
}

void main()
{
   float attenuation;
   attenuation = superEllipseShape(vPosLC) *
                   distanceAttenuation(vPosLC, 2.0, 1.0, 1.0 );

   vec3 N = normalize(normLC);
   vec3 L = -normalize(vPosLC);
   vec3 V = normalize(eyeLC-vPosLC);
   vec3 H = normalize(L + V);   // half-vector
      
   vec3 baseColor = vec3(1.0, 1.0, 1.0);
   float shinness = 4.0;
   float NdotL = dot(N, L);
   float NdotH = dot(N, H);

   float diff = max(NdotL, 0.0);
   float spec =  max(NdotH, 0.0); 
   
   vec3 ambient = vec3 (0.5, 0.2, 0.2);
   vec3 diffuse = attenuation * baseColor  * diff;
   vec3 specular = attenuation * baseColor * pow(spec, shinness);
      
   gl_FragColor = vec4(diffuse + specular + ambient, 1.0);
}
